@@ -15,7 +15,7 @@
 
 # lbaasv2 driver for MidoNet
 
-from neutron import context as ncontext
+from neutron_lib import context as ncontext
 from neutron_lib.plugins import directory
 
 from oslo_config import cfg
@@ -28,11 +28,16 @@ from neutron_lbaas.drivers import driver_base
 from neutron_lbaas.services.loadbalancer import constants as lb_const
 
 from midonet.neutron.client import base as c_base
+from midonet.neutron.common import config  # noqa
 from midonet.neutron.common import utils
 
 LOG = logging.getLogger(__name__)
 
 
+# TODO(yamamoto): Introduce LBaaS PRECOMMIT callbacks and
+# subscribe them for task-based api.
+
+
 MN_STATUS_TO_OPERATING_STATUS = {
     'ACTIVE': lb_const.ONLINE,
     'INACTIVE': lb_const.OFFLINE,
@@ -45,10 +50,15 @@ class MidonetLoadBalancerDriver(driver_base.LoadBalancerBaseDriver):
     def __init__(self, plugin):
         super(MidonetLoadBalancerDriver, self).__init__(plugin)
 
+        # pylint: disable-msg=E0110
         self.load_balancer = MidonetLoadBalancerManager(self)
+        # pylint: disable-msg=E0110
         self.listener = MidonetListenerManager(self)
+        # pylint: disable-msg=E0110
         self.pool = MidonetPoolManager(self)
+        # pylint: disable-msg=E0110
         self.member = MidonetMemberManager(self)
+        # pylint: disable-msg=E0110
         self.health_monitor = MidonetHealthMonitorManager(self)
         self._client = c_base.load_client(cfg.CONF.MIDONET)
 
